local modenv = require "unik.modenv"
local MakePlayerCharacter = require "prefabs/player_common"

local assets = {
        Asset( "ANIM", "anim/unik.zip" ),
        Asset( "ANIM", "anim/unik_ins.zip" ),
        Asset( "ANIM", "anim/unik_bus.zip" ),
        Asset( "ANIM", "anim/unik_puk.zip" ),
        Asset( "ANIM", "anim/ghost_unik_build.zip" ),
}
local prefabs = {}

local unik_health_start = modenv.GetModConfigData("health_start")
local unik_health_max = modenv.GetModConfigData("health_max")
local unik_sanity_start = modenv.GetModConfigData("sanity_start")
local unik_sanity_max = modenv.GetModConfigData("sanity_max")
local unik_hunger_start = modenv.GetModConfigData("hunger_start")
local unik_hunger_max = modenv.GetModConfigData("hunger_max")
local unik_angry_kitty_sanity = modenv.GetModConfigData("angry_kitty_sanity")
local unik_biznis_kitty_hunger = modenv.GetModConfigData("biznis_kitty_hunger")
local unik_queasy_kitty_hunger = modenv.GetModConfigData("queasy_kitty_hunger")
local biznis_kitty_speed = modenv.GetModConfigData("biznis_kitty_speed")
local unikitty_speed = modenv.GetModConfigData("unikitty_speed")
local angry_kitty_speed = modenv.GetModConfigData("angry_kitty_speed")
local queasy_kitty_speed = modenv.GetModConfigData("queasy_kitty_speed")
local unik_ghost_speed = modenv.GetModConfigData("ghost_speed")
local unikitty_damage_multiplier = modenv.GetModConfigData("unikitty_damage_multiplier")
local angry_kitty_damage_multiplier = modenv.GetModConfigData("angry_kitty_damage_multiplier")
local angry_kitty_hunger_rate = modenv.GetModConfigData("angry_kitty_hunger_rate")

local function applymightiness(inst)
    if inst.strength == "normal" then
        inst.AnimState:SetBuild("unik")
        inst.components.locomotor.walkspeed = unikitty_speed * 0.75
        inst.components.locomotor.runspeed = unikitty_speed 
    elseif inst.strength == "mighty" then
        inst.AnimState:SetBuild("unik_bus")
        inst.components.locomotor.walkspeed = biznis_kitty_speed * 0.75
        inst.components.locomotor.runspeed = biznis_kitty_speed 
    elseif inst.strength == "wimpy" then
        inst.AnimState:SetBuild("unik_puk") 
        inst.components.locomotor.walkspeed = queasy_kitty_speed * 0.75
        inst.components.locomotor.runspeed = queasy_kitty_speed
    end
end

local function updateHungerState(inst)
    local silent = POPULATING
    local mightyTrigger = unik_hunger_max * unik_biznis_kitty_hunger
    local wimpyTrigger = unik_hunger_max * unik_queasy_kitty_hunger
    
    if inst.strength == "mighty" then
        if inst.components.hunger.current < mightyTrigger then
            inst.strength = "normal"
            if not silent then
                inst.sg:PushEvent("powerdown")            
                inst.components.talker:Say("I feel a bit hungry, this is not silly business!")
            end
        end
    elseif inst.strength == "wimpy" then
        if inst.components.hunger.current > wimpyTrigger then
            inst.strength = "normal"
            if not silent then
                inst.sg:PushEvent("powerup")
                inst.components.talker:Say("I feel better now!")                
            end
        end
    else
        if inst.components.hunger.current > mightyTrigger then
            inst.strength = "mighty"
            if not silent then
                inst.sg:PushEvent("powerup")
                inst.components.talker:Say("Time for bizniz!")                
            end
        elseif inst.components.hunger.current < wimpyTrigger then
            inst.strength = "wimpy"
            if not silent then
                inst.sg:PushEvent("powerdown")
                inst.components.talker:Say("I'm so hungry I could puke!")
            end
        end
    end
end

local function updateUnikitty(inst, data)
    local silent = POPULATING
    local currentSanity = inst.components.sanity.current
    local angryKittyTrigger = unik_angry_kitty_sanity * unik_sanity_max
    
    if inst:HasTag("playerghost") or
        inst.components.health:IsDead() then
        return
    end

    if inst.insane == "false" then
        if currentSanity == angryKittyTrigger and inst.stayPositive == "false" then
            if not silent then
                inst.stayPositive = "true"
                inst.components.talker:Say("Stay positive!")
            end
        end
        
        if currentSanity < angryKittyTrigger then
            inst.insane = "true"
            inst.stayPositive = "false"
            inst.AnimState:SetBuild("unik_ins")
            inst.components.combat.damagemultiplier = angry_kitty_damage_multiplier
            inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * angry_kitty_hunger_rate)
            inst.components.locomotor.walkspeed = angry_kitty_speed * 0.75
            inst.components.locomotor.runspeed = angry_kitty_speed 
            if not silent then
                inst.components.talker:Say("Oh, forget it! Rahhh!")
            end
        else
            updateHungerState(inst)
            applymightiness(inst)
        end
    else
        if currentSanity >= angryKittyTrigger then
            inst.insane = "false"
            inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE)
            inst.components.combat.damagemultiplier = unikitty_damage_multiplier
            if not silent then
                inst.components.talker:Say("I must stay positive! Bubble gums!")
            end
            updateHungerState(inst)
            applymightiness(inst) 
        end
    end
end

local function onbecamehuman(inst, data)
    inst.insane = "false"
    inst.stayPositive = "false"
    inst.components.combat.damagemultiplier = unikitty_damage_multiplier
    inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE)

    inst:ListenForEvent("hungerdelta", updateUnikitty)
    inst:ListenForEvent("sanitydelta", updateUnikitty)
end

local function onbecameghost( inst, data )
    inst:RemoveEventCallback("hungerdelta", updateUnikitty)    
    inst:RemoveEventCallback("sanitydelta", updateUnikitty)        
    
    inst.components.locomotor.walkspeed = unik_ghost_speed * 0.75
    inst.components.locomotor.runspeed = unik_ghost_speed
end

local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end

-- This initializes for both clients and the host
local function common_postinit(inst)
    -- choose which sounds this character will play
    inst.soundsname = "willow"

    inst.avatar_tex   = "avatar_unik.tex"
    inst.avatar_atlas = "images/avatars/avatar_unik.xml"
    inst.avatar_ghost_tex   = "avatar_ghost_unik.tex"
    inst.avatar_ghost_atlas = "images/avatars/avatar_ghost_unik.xml"
    
    -- a minimap icon must be specified
    inst.MiniMapEntity:SetIcon( "unik.tex" )
end

-- This initializes for the host only
local function master_postinit(inst)
    inst.strength = "normal"

    inst.insane = "false"
    inst.stayPositive = "false"

    -- set stats
    inst.components.health.maxhealth = unik_health_max
    inst.components.health:SetPercent( unik_health_start )

    inst.components.sanity:SetMax( unik_sanity_max )
    inst.components.sanity:SetPercent( unik_sanity_start )

    inst.components.hunger:SetMax( unik_hunger_max )
--    inst.components.hunger:SetPercent( unik_hunger_start )

    inst.OnLoad = onload
    inst.OnNewSpawn = onload
end

STRINGS.CHARACTERS.UNIK = require "speech_unik"
table.insert(CHARACTER_GENDERS.FEMALE, "unik")

return MakePlayerCharacter("unik", prefabs, assets, common_postinit, master_postinit)